<?php

namespace Util;

use Doctrine\Common\Cache\Cache as CacheInterface;
use Zeedhi\Framework\Cache\Cache;
use Zeedhi\Framework\Cache\Type\MongoImpl;
use Exception\Token as TokenException;

class MongoImplCustom extends MongoImpl implements Cache, CacheInterface
{
    /**
     * {@inheritdoc}
     */
    public function fetch($key, $lifeTime = 0, $keepConnected = false) {
        $documents = $this->mongo->find($this->getCollectionName(), array('_id' => $key));
        $document = array_shift($documents);

        if ($document === null) {
            TokenException::invalidToken($key);
        }

        if ($this->isExpired($document)) {
            if($keepConnected) {
                $params = unserialize($document[self::DATA_FIELD]->bin);
                $this->save($key, $params, $lifeTime);
            } else {
                $this->delete($key);
                TokenException::expiredToken($key);
            }
        }

        return unserialize($document[self::DATA_FIELD]->bin);
    }

    /**
     * Check if the document is expired.
     *
     * @param array $document
     *
     * @return boolean
     */
    protected function isExpired(array $document) {
        return isset($document[self::EXPIRATION_FIELD]) &&
               $document[self::EXPIRATION_FIELD] instanceof \MongoDate &&
               $document[self::EXPIRATION_FIELD]->sec < time();
    }
}